using UnityEngine;
using System.Collections;
using System.Collections.Generic;

/// <summary>
/// 	A basic inventory system that can be used pick up, drop and use items found in the game world.
/// </summary>
/// <remarks>
/// 	This system uses a Unity Asset Store package called ex2D to do all the 2D sprite drawing/atlasing.
/// 
/// 	Author: Scott Beca, Considerable Content, 2012
/// </remarks>
public class Inventory : MonoBehaviour {
	public static Inventory Instance;
	
	public int GuiLayer = 8;
	public List<InventoryItem> InventoryItems;
	public int SelectedItem = -1;
	public Camera GuiCamera = null;
	public exAtlas InventoryAtlas = null;
	public Texture InventorySingle = null;
	public Texture InventoryLeft = null;
	public Texture InventoryMiddle = null;
	public Texture InventoryRight = null;
	public Texture InventorySelected = null;
	
	private GameObject inventoryGameObject;
	private GameObject inventorySelectedGameObject;
	private bool inventorySwitchDelayed = false;
	
	private bool animatingItem = false;
	private exScreenPosition animatedItem;
	private Vector2 animationStartPoint;
	private Vector2 animationEndPoint;
	private float animationStartTime;
	private float animationDuration = 0.1f;
	
	public delegate void InventoryEvent(object o, System.EventArgs e);
	public event InventoryEvent InventoryBringOut;
	public event InventoryEvent InventoryPutAway;
	public event InventoryEvent InventoryUse;
	
	[System.Serializable]
	public class InventoryItem {
		public string ObjectName;
		public GameObject WorldObject;
		public Texture GuiTexture;
		public int InventoryPosition = -1;
		public bool PickedUp = false;
		public bool Usable = false;
		
		[HideInInspector]
		public GameObject inventoryObject;
	}
	
	void Awake() {
		Instance = this;
		
		Transform t;
		if (t = GuiCamera.transform.FindChild("Inventory")) {
			inventoryGameObject = t.gameObject;
		} else {
			inventoryGameObject = new GameObject("Inventory");
			inventoryGameObject.transform.parent = GuiCamera.transform;
		}
		
		inventoryGameObject.transform.localPosition = Vector3.zero;
		
		// Create all the sprites that will be used by the inventory in this level
		if (InventoryItems.Count == 1) {
			CreateInventorySpot("InventorySingle", InventorySingle, 5, 30);
			
		} else if (InventoryItems.Count > 1) {
			CreateInventorySpot("InventoryLeft", InventoryLeft, 5, 30);
			
			int i;
			for (i = 0; i < InventoryItems.Count - 2; i++) {
				CreateInventorySpot("InventoryMiddle" + i, InventoryMiddle, (InventoryLeft.width - 1) + 5 + i * (InventoryMiddle.width - 1), 30);
			}
			
			CreateInventorySpot("InventoryRight", InventoryRight, (InventoryLeft.width - 1) + 5 + i * (InventoryMiddle.width - 1), 30);
		}
		
		GameObject itemObject;
		foreach (InventoryItem item in InventoryItems) {
			if (item.InventoryPosition == 0) {
				itemObject = CreateInventorySpot(item.ObjectName, item.GuiTexture, 5, 20);
				item.inventoryObject = itemObject;
				
				if (!item.PickedUp) {
					itemObject.renderer.enabled = false;
				}
			} else if (item.InventoryPosition > 0) {
				itemObject = CreateInventorySpot(item.ObjectName, item.GuiTexture, (InventoryLeft.width - 1) + 4 + (item.InventoryPosition - 1) * (InventoryMiddle.width - 1), 20);
				item.inventoryObject = itemObject;
				
				if (!item.PickedUp) {
					itemObject.renderer.enabled = false;
				}
			}
			
			if (item.WorldObject) {
				ToggleWorldObject(item.WorldObject, false);
			}
		}
		
		if (SelectedItem < 0) {
			inventorySelectedGameObject = CreateInventorySpot("Inventory_Selected", InventorySelected, 5, 10);
			inventorySelectedGameObject.renderer.enabled = false;
		} else if (SelectedItem == 0) {
			inventorySelectedGameObject = CreateInventorySpot("Inventory_Selected", InventorySelected, 5, 10);
		} else {
			inventorySelectedGameObject = CreateInventorySpot("Inventory_Selected", InventorySelected, (InventoryLeft.width - 1) + 3 + (SelectedItem - 1) * (InventoryMiddle.width - 1), 10);
		}
		
		if (!InventoryItems[SelectedItem].PickedUp) {
			SelectItem(GetPrevItemIndex());
		}
	}

	void Update() {
		// Get player inputs related to the inventory
		if (!inventorySwitchDelayed) {
			if (Input.GetAxis("Mouse ScrollWheel") < 0) {
				SelectItem(GetPrevItemIndex());
				inventorySwitchDelayed = true;
				StartCoroutine(ResetInventorySwitchDelay());
			} else if (Input.GetAxis("Mouse ScrollWheel") > 0) {
				SelectItem(GetNextItemIndex());
				inventorySwitchDelayed = true;
				StartCoroutine(ResetInventorySwitchDelay());
			}
			
			for (int i = 1; i < InventoryItems.Count + 1; i++) {
				if (Input.GetKeyDown(i.ToString())) {
					SelectItem(i - 1);
					inventorySwitchDelayed = true;
					StartCoroutine(ResetInventorySwitchDelay());
				}
			}
		}
		
		if (Input.GetButtonDown("Fire1")) {
			if (InventoryUse != null) {
				InventoryUse(this, new System.EventArgs());
			}
		}
		
		// Animate the picking up of an item
		if (animatingItem) {
			Vector2 newPos = Vector2.Lerp(animationStartPoint, animationEndPoint, (Time.time - animationStartTime) / animationDuration);
			animatedItem.x = newPos.x;
			animatedItem.y = newPos.y;
			
			if (Time.time > animationStartTime + animationDuration) {
				animatingItem = false;
			}
		}
	}
	
	public int GetSelectedIndex() {
		return SelectedItem;
	}
	
	public bool SelectItem(int index) {
		bool result = false;
		
		if (index >= 0 && index < InventoryItems.Count && InventoryItems[index].PickedUp) {
			
			if (InventoryPutAway != null) {
				InventoryPutAway(this, new System.EventArgs());
			}
			if (!inventorySelectedGameObject.renderer.enabled) {
				inventorySelectedGameObject.renderer.enabled = true;
			}
			
			if (SelectedItem > 0 && InventoryItems[SelectedItem].WorldObject) {
				ToggleWorldObject(InventoryItems[SelectedItem].WorldObject, false);
			}
			
			SelectedItem = index;
			
			exScreenPosition screenPos = inventorySelectedGameObject.GetComponent<exScreenPosition>();
			if (SelectedItem == 0) {
				screenPos.x = 5;
			} else {
				screenPos.x = (InventoryLeft.width - 1) + 3 + (SelectedItem - 1) * (InventoryMiddle.width - 1);
			}
			
			if (InventoryItems[SelectedItem].Usable && InventoryItems[SelectedItem].WorldObject) {
				ToggleWorldObject(InventoryItems[SelectedItem].WorldObject, true);
			}
			
			if (InventoryBringOut != null) {
				InventoryBringOut(this, new System.EventArgs());
			}
			
			result = true;
		} else if (index == -1) {
			if (inventorySelectedGameObject.renderer.enabled) {
				inventorySelectedGameObject.renderer.enabled = false;
			}
			SelectedItem = -1;
		}
		
		return result;
	}
	
	public void PickupItem(int index) {
		if (index >= 0 && index < InventoryItems.Count && !InventoryItems[index].PickedUp) {
			exScreenPosition screenPos = InventoryItems[index].inventoryObject.GetComponent<exScreenPosition>();
			
			animationEndPoint = new Vector2(screenPos.x, screenPos.y);
			
			screenPos.x = Screen.width / 2 - InventoryItems[index].GuiTexture.width / 2;
			screenPos.y = Screen.height / 2 - InventoryItems[index].GuiTexture.height / 2;
			InventoryItems[index].PickedUp = true;
			InventoryItems[index].inventoryObject.renderer.enabled = true;
			
			animationStartPoint = new Vector2(screenPos.x, screenPos.y);
    		animationStartTime = Time.time;
			animatedItem = screenPos;
			animatingItem = true;
			
			Debug.Log(index);
			if (SelectedItem < 0) {
				SelectItem(index);
			}
		}
	}
	
	public void DropItem(int index) {
		if (index >= 0 && index < InventoryItems.Count && InventoryItems[index].PickedUp) {
			InventoryItems[index].PickedUp = false;
			InventoryItems[index].inventoryObject.renderer.enabled = false;
			
			if (InventoryItems[index].WorldObject) {
				ToggleWorldObject(InventoryItems[index].WorldObject, false);
			}
			
			SelectItem(GetPrevItemIndex());
		}
	}
	
	private IEnumerator ResetInventorySwitchDelay() {
		yield return new WaitForSeconds(0.1f);
		inventorySwitchDelayed = false;
	}
	
	private GameObject CreateInventorySpot(string name, Texture tex, int x, int z) {
		GameObject newGameObject = new GameObject(name);
		newGameObject.transform.parent = inventoryGameObject.transform;
		newGameObject.layer = GuiLayer;
		newGameObject.AddComponent<exSprite>();
		
		exSprite sp = newGameObject.GetComponent<exSprite>();
		
		int index = InventoryAtlas.GetIndexByName(tex.name);
		sp.SetSprite(InventoryAtlas, index);
		
		sp.renderCamera = GuiCamera;
		sp.anchor = exPlane.Anchor.BotLeft;
		
		newGameObject.AddComponent<exPixelPerfect>();
		exScreenPosition screenPos = newGameObject.AddComponent<exScreenPosition>();
		
		Vector3 spritePos = newGameObject.transform.position;
		spritePos.z = z;
		newGameObject.transform.localPosition = spritePos;
		
		screenPos.x = x;
		screenPos.y = 5;
		
		return newGameObject;
	}
	
	private int GetNextItemIndex() {
		int test = SelectedItem + 1;
		if (test >= InventoryItems.Count) {
			test = 0;
		}
		
		for (int i = 0; i < InventoryItems.Count; i++) {
			if (InventoryItems[test].PickedUp) {
				return test;
			} 
			
			test++;
			if (test >= InventoryItems.Count) {
				test = 0;
			}
		}
		
		return -1;
	}
	
	private int GetPrevItemIndex() {
		int test = SelectedItem - 1;
		if (test < 0) {
			test = InventoryItems.Count - 1;
		}
		
		for (int i = 0; i < InventoryItems.Count; i++) {
			if (InventoryItems[test].PickedUp) {
				return test;
			} 
			
			test--;
			if (test < 0) {
				test = InventoryItems.Count - 1;
			}
		}
		
		return -1;
	}
	
	private void ToggleWorldObject(GameObject worldObject, bool show) {
		if (worldObject.active != show) {
			worldObject.active = show;
			worldObject.SetActiveRecursively(show);
		}
	}
}
